# -*- coding: utf-8 -*-
"""
app.main.forms
~~~~~~~~~~~~~~

This module defines the main forms.

:copyright: (c) 2017 European Spallation Source ERIC
:license: BSD 2-Clause, see LICENSE for more details.

"""
from flask_wtf import FlaskForm
from wtforms import SelectField, StringField, SelectMultipleField, validators
from .. import utils, models


class NoValidateSelectField(SelectField):
    """SelectField with no choices validation

    By default a SelectField tries to validate the selected value
    against the list of choices. This is not possible when the choices
    are dynamically created on the browser side.
    """

    def pre_validate(self, form):
        pass


class AttributeForm(FlaskForm):
    name = StringField('name', validators=[validators.DataRequired()])


class ItemForm(FlaskForm):
    ics_id = StringField('ICS id',
                         validators=[validators.InputRequired(),
                                     validators.Regexp(models.ICS_ID_RE)])
    serial_number = StringField('Serial number',
                                validators=[validators.InputRequired()])
    manufacturer_id = SelectField('Manufacturer')
    model_id = SelectField('Model')
    location_id = SelectField('Location')
    status_id = SelectField('Status')
    parent_id = SelectField('Parent')
    # macs = SelectMultipleField('Macs')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.manufacturer_id.choices = utils.get_model_choices(models.Manufacturer, allow_blank=True)
        self.model_id.choices = utils.get_model_choices(models.Model, allow_blank=True)
        self.location_id.choices = utils.get_model_choices(models.Location, allow_blank=True)
        self.status_id.choices = utils.get_model_choices(models.Status, allow_blank=True)
        self.parent_id.choices = utils.get_model_choices(models.Item, allow_blank=True, attr='ics_id')


class HostForm(FlaskForm):
    network_id = SelectField('Network')
    # The list of IPs is dynamically created on the browser side
    # depending on the selected network
    ip = NoValidateSelectField('IP', choices=[])
    name = StringField('Hostname',
                       description='hostname must be 2-20 characters long and contain only letters, numbers and dash',
                       validators=[validators.InputRequired(),
                                   validators.Regexp(models.HOST_NAME_RE)],
                       filters=[utils.lowercase_field])
    description = StringField('Description')
    mac_id = SelectField('MAC')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.network_id.choices = [(str(network.id), network.vlan_name) for network in models.Network.query.all()]
        self.mac_id.choices = [('', '')]
        self.mac_id.choices.extend([(str(mac.id), mac.address) for mac in models.Mac.query.all()])
