# -*- coding: utf-8 -*-
"""
app.main.views
~~~~~~~~~~~~~~

This module implements the main blueprint.

:copyright: (c) 2017 European Spallation Source ERIC
:license: BSD 2-Clause, see LICENSE for more details.

"""
import sqlalchemy as sa
from flask import (Blueprint, render_template, jsonify, session,
                   redirect, url_for, request, flash, current_app)
from flask_login import login_required
from .forms import AttributeForm, HostForm, ItemForm
from ..extensions import db
from ..decorators import login_groups_accepted
from .. import utils, models

bp = Blueprint('main', __name__)


# Declare custom error handlers for all views
@bp.app_errorhandler(404)
def not_found_error(error):
    return render_template('404.html'), 404


@bp.app_errorhandler(500)
def internal_error(error):
    return render_template('500.html'), 500


@bp.app_errorhandler(utils.CSEntryError)
def handle_csentry_error(error):
    response = jsonify(error.to_dict())
    response.status_code = error.status_code
    return response


@bp.route('/_retrieve_items')
@login_required
def retrieve_items():
    items = models.Item.query.order_by(models.Item._created)
    data = [[item.id,
             item.ics_id,
             utils.format_field(item._created),
             utils.format_field(item._updated),
             item.serial_number,
             utils.format_field(item.manufacturer),
             utils.format_field(item.model),
             utils.format_field(item.location),
             utils.format_field(item.status),
             utils.format_field(item.parent),
             ] for item in items]
    return jsonify(data=data)


@bp.route('/')
@login_required
def index():
    return render_template('index.html')


@bp.route('/items')
@login_required
def list_items():
    return render_template('items.html')


@bp.route('/items/create', methods=('GET', 'POST'))
@login_groups_accepted('admin', 'create')
def create_item():
    form = ItemForm()
    if form.validate_on_submit():
        item = models.Item(ics_id=form.ics_id.data,
                           serial_number=form.serial_number.data,
                           manufacturer_id=form.manufacturer_id.data or None,
                           model_id=form.model_id.data or None,
                           location_id=form.location_id.data or None,
                           status_id=form.status_id.data or None,
                           parent_id=form.parent_id.data or None)
        current_app.logger.debug(f'Trying to create: {item!r}')
        db.session.add(item)
        try:
            db.session.commit()
        except sa.exc.IntegrityError as e:
            db.session.rollback()
            current_app.logger.warning(f'{e}')
            flash(f'{e}', 'error')
        else:
            flash(f'Item {item} created!', 'success')
        return redirect(url_for('main.create_item'))
    return render_template('create_item.html', form=form)


@bp.route('/items/view/<ics_id>')
@login_required
def view_item(ics_id):
    item = models.Item.query.filter_by(ics_id=ics_id).first_or_404()
    return render_template('view_item.html', item=item.to_dict(long=True))


@bp.route('/qrcodes/<kind>')
@login_required
def qrcodes(kind='Action'):
    try:
        model = getattr(models, kind)
    except AttributeError:
        raise utils.CSEntryError(f"Unknown model '{kind}'", status_code=422)
    items = db.session.query(model).order_by(model.name)
    images = [{'name': item.name,
               'data': utils.image_to_base64(item.image())}
              for item in items]
    return render_template('qrcodes.html', kind=kind, images=images)


@bp.route('/attributes/<kind>', methods=('GET', 'POST'))
@login_groups_accepted('admin', 'create')
def attributes(kind):
    form = AttributeForm()
    if form.validate_on_submit():
        model = getattr(models, kind)
        new_model = model(name=form.name.data)
        db.session.add(new_model)
        try:
            db.session.commit()
        except sa.exc.IntegrityError as e:
            db.session.rollback()
            flash(f'{form.name.data} already exists! {kind} not created.', 'error')
        else:
            flash(f'{kind} {new_model} created!', 'success')
        return redirect(url_for('main.attributes', kind=kind))
    return render_template('attributes.html', kind=kind, form=form)


@bp.route('/_retrieve_attributes_name/<kind>')
@login_required
def retrieve_attributes_name(kind):
    try:
        model = getattr(models, kind)
    except AttributeError:
        raise utils.CSEntryError(f"Unknown model '{kind}'", status_code=422)
    items = db.session.query(model).order_by(model.name)
    data = [[item.name] for item in items]
    return jsonify(data=data)


@bp.route('/hosts')
@login_required
def list_hosts():
    return render_template('hosts.html')


@bp.route('/hosts/create', methods=('GET', 'POST'))
@login_groups_accepted('admin', 'create')
def create_host():
    # Try to get the network_id from the session
    # to pre-fill the form with the same network
    try:
        network_id = session['network_id']
    except KeyError:
        # No need to pass request.form when no extra keywords are given
        form = HostForm()
    else:
        form = HostForm(request.form, network_id=network_id)
    if form.validate_on_submit():
        network_id = form.network_id.data
        host = models.Host(ip=form.ip.data,
                           network_id=form.network_id.data,
                           name=form.name.data,
                           description=form.description.data or None,
                           mac_id=form.mac_id.data or None)
        current_app.logger.debug(f'Trying to create: {host!r}')
        db.session.add(host)
        try:
            db.session.commit()
        except sa.exc.IntegrityError as e:
            db.session.rollback()
            current_app.logger.warning(f'{e}')
            flash(f'{e}', 'error')
        else:
            flash(f'Host {host} created!', 'success')
        # Save network_id to the session to retrieve it after the redirect
        session['network_id'] = host.network_id
        return redirect(url_for('main.create_host'))
    return render_template('create_host.html', form=form)


@bp.route('/_retrieve_hosts')
@login_required
def retrieve_hosts():
    data = [(host.name,
             host.ip,
             host.description,
             str(host.mac),
             host.network.vlan_name) for host in models.Host.query.all()]
    return jsonify(data=data)


@bp.route('/_retrieve_available_ips/<network_id>')
@login_required
def retrieve_available_ips(network_id):
    try:
        network = models.Network.query.get(network_id)
    except sa.exc.DataError:
        current_app.logger.warning(f'Invalid network_id: {network_id}')
        data = []
    else:
        data = [str(address) for address in network.available_ips()]
    return jsonify(data=data)
